/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import Foundation
import AppKit

import BluetoothControl
import CoreBluetooth
import CameraControlInterface

class BaseViewController: NSViewController, ConnectionStatusToUIDelegate {

	// Member variables
	var m_selectController: SelectViewController?
    var m_contentController: ContentViewController?
    var m_mainController: MainViewController?
    var m_slateController: SlateViewController?

	//==================================================
	//	UIViewController methods
	//==================================================
    override func viewDidLoad() {
        let appDelegate = NSApplication.shared.delegate as! AppDelegate
        let cameraControlInterface: CameraControlInterface = appDelegate.cameraControlInterface
        cameraControlInterface.m_connectionStatusDelegate = self

        let mainStoryboard: NSStoryboard = NSStoryboard(name: "Main", bundle: nil)
        m_selectController = mainStoryboard.instantiateController(withIdentifier: "selectController") as? SelectViewController
        m_contentController = mainStoryboard.instantiateController(withIdentifier: "contentController") as? ContentViewController

        showSelectView()

        super.viewDidLoad()
    }

	//==================================================
	//	ConnectionStatusToUIDelegate methods
	//==================================================
    func onConnectionLost() {
		switchToSelectView()
		PresentModal(title: String.Localized("DisconnectedModal.Title"), subtitle: String.Localized("DisconnectedModal.Subtitle"))
    }

    func onDisconnection() {
        switchToSelectView()
    }

	func onReconnection() {}
    func onCameraPoweredOff() {}
    func onCameraPoweredOn() {}
    func onCameraReady() {}
	
	//==================================================
	//	UI control
	//==================================================
	func showSelectView() {
		if let selectController = m_selectController {
			addChild(selectController)
			view.addSubview(selectController.view)
			view.frame = selectController.view.frame
		}
	}
	
	func removeSelectView() {
		if let selectController = m_selectController {
			removeChild(at: 0)
			selectController.view.removeFromSuperview()
		}
	}
	
	func showContentView() {
		if let contentController = m_contentController {
			addChild(contentController)
			view.addSubview(contentController.view)
			view.frame = contentController.view.frame
		}
	}
	
	func removeContentView() {
		if let contentController = m_contentController {
			removeChild(at: 0)
			contentController.view.removeFromSuperview()
		}
	}
	
	func switchToSelectView() {
		removeContentView()
		showSelectView()
	}
	
	func switchToContentView() {
		removeSelectView()
		showContentView()
	}
}
