/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import AppKit
import BluetoothControl
import Utility
import CCU
import CameraControlInterface

class ISOButtonViewController: NSViewController {

	// IBOutlets
	@IBOutlet weak var iso200Button: NSButton!
	@IBOutlet weak var iso400Button: NSButton!
	@IBOutlet weak var iso800Button: NSButton!
	@IBOutlet weak var iso1600Button: NSButton!

	// Member variables
	weak var cciOut: OutgoingCameraControlFromUIDelegate?
	var buttons = [NSButton]()
	var isoValues = VideoConfig.kISOValues[CameraModel.Unknown]!
	
	//==================================================
	//	UIViewController methods
	//==================================================
	override func viewDidLoad() {
		super.viewDidLoad()

		buttons.append(iso200Button)
		buttons.append(iso400Button)
		buttons.append(iso800Button)
		buttons.append(iso1600Button)

		updateButtons()

		let appDelegate = NSApplication.shared.delegate as! AppDelegate
		cciOut = appDelegate.cameraControlInterface
	}

	//==================================================
	//	IBActions
	//==================================================
	@IBAction func onISOButtonClicked(_ sender: NSButton) {
		let isoIndex: Int = sender.tag
		cciOut?.onISOChanged(isoIndex)
	}

	//==================================================
	//	IncomingCameraControlToUIDelegate methods
	//==================================================
	func onISOReceived(_ isoIndex: Int) {
		buttons.forEach({$0.isSelected = $0.tag == isoIndex})
	}
	
	func onCameraSpecificationReceived(_ cameraModel: CameraModel) {
		isoValues = VideoConfig.kISOValues[cameraModel] ?? VideoConfig.kISOValues[CameraModel.Unknown]!
		updateButtons()
	}

	func updateButtons() {
		for button in buttons {
			if let iso = button.title.toInt() {
				if let index = isoValues.firstIndex(of: iso) {
					button.tag = index
				}
			}
		}
	}
}
