/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import Foundation
import AppKit
import BluetoothControl
import CameraControlInterface

class SlateViewController: NSViewController, IncomingSlateToUIDelegate {

    // IBOutlets
    @IBOutlet var sceneName: NSTextField!
    @IBOutlet var reelLabel: NSTextField!
    @IBOutlet var sceneLabel: NSTextField!
    @IBOutlet var takeLabel: NSTextField!

    @IBOutlet weak var sceneTagWSButton: NSButton!
    @IBOutlet weak var sceneTagCUButton: NSButton!
    @IBOutlet weak var sceneTagMSButton: NSButton!
    @IBOutlet weak var sceneTagBCUButton: NSButton!
    @IBOutlet weak var sceneTagMCUButton: NSButton!
    @IBOutlet weak var sceneTagECUButton: NSButton!
	
    @IBOutlet weak var takeTagPUButton: NSButton!
    @IBOutlet weak var takeTagVFXButton: NSButton!
    @IBOutlet weak var takeTagSERButton: NSButton!

    @IBOutlet weak var goodTakeButton: NSButton!

    @IBOutlet weak var locationIntRadioButton: NSButton!
    @IBOutlet weak var locationExtRadioButton: NSButton!
	
    @IBOutlet weak var timeDayRadioButton: NSButton!
    @IBOutlet weak var timeNightRadioButton: NSButton!

    @IBOutlet weak var reelPrevButton: NSButton!
    @IBOutlet weak var reelNextButton: NSButton!

	// Member variables
	weak var m_outgoingSlateDelegate: OutgoingSlateFromUIDelegate?
	var sceneTagButtons = [NSButton]()
	var takeTagButtons = [NSButton]()
	var locationRadioButtons = [NSButton]()
    var timeRadioButtons = [NSButton]()

	//==================================================
	//	UIViewController methods
	//==================================================
    override func viewDidLoad() {
        sceneTagButtons.append(sceneTagWSButton)
        sceneTagButtons.append(sceneTagCUButton)
        sceneTagButtons.append(sceneTagMSButton)
        sceneTagButtons.append(sceneTagBCUButton)
        sceneTagButtons.append(sceneTagMCUButton)
        sceneTagButtons.append(sceneTagECUButton)

        takeTagButtons.append(takeTagPUButton)
        takeTagButtons.append(takeTagVFXButton)
        takeTagButtons.append(takeTagSERButton)

        locationRadioButtons.append(locationIntRadioButton)
        locationRadioButtons.append(locationExtRadioButton)

        timeRadioButtons.append(timeDayRadioButton)
        timeRadioButtons.append(timeNightRadioButton)

		// Register as a IncomingSlateToUIDelegate with the CameraControlInterface,
		// and assign CameraControlInterface as our OutgoingSlateFromUIDelegate.
        let appDelegate = NSApplication.shared.delegate as! AppDelegate
        let cameraControlInterface: CameraControlInterface = appDelegate.cameraControlInterface
        m_outgoingSlateDelegate = cameraControlInterface
        cameraControlInterface.m_slateToUIDelegate = self
        cameraControlInterface.onSlateViewLoaded()
    }

	//==================================================
	//	IBActions
	//==================================================
    @IBAction func onPrevReelButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onReelDecremented()
    }

    @IBAction func onNextReelButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onReelIncremented()
    }

    @IBAction func onPrevSceneButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onSceneDecremented()
    }

    @IBAction func onNextSceneButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onSceneIncremented()
    }

    @IBAction func onPrevTakeButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onTakeDecremented()
    }

    @IBAction func onNextTakeButtonClicked(_: NSButton) {
        m_outgoingSlateDelegate?.onTakeIncremented()
    }

    @IBAction func onSceneTagButtonClicked(_ sender: NSButton) {
        let newState = sender.state == NSControl.StateValue.on ? NSControl.StateValue.on : NSControl.StateValue.off
        sceneTagButtons.forEach { $0.state = NSControl.StateValue.off }
        sceneTagButtons[sender.tag].state = newState

        if newState == NSControl.StateValue.on {
            m_outgoingSlateDelegate?.onSceneTagSelected(sceneTagIndex: sender.tag)
        } else {
            m_outgoingSlateDelegate?.onSceneTagSelected(sceneTagIndex: -1)
        }
    }

    @IBAction func onTakeTagButtonClicked(_ sender: NSButton) {
        let newState = sender.state == NSControl.StateValue.on ? NSControl.StateValue.on : NSControl.StateValue.off
        takeTagButtons.forEach { $0.state = NSControl.StateValue.off }
        takeTagButtons[sender.tag].state = newState

        if newState == NSControl.StateValue.on {
            m_outgoingSlateDelegate?.onTakeTagSelected(takeTagIndex: sender.tag)
        } else {
            m_outgoingSlateDelegate?.onTakeTagSelected(takeTagIndex: -1)
        }
    }

    @IBAction func onGoodTakeButtonClicked(_ sender: NSButton) {
        m_outgoingSlateDelegate?.onGoodTakeToggled(sender.state == NSControl.StateValue.on)
    }

    @IBAction func onLocationRadioButtonClicked(_ sender: NSButton) {
        locationRadioButtons.forEach { $0.state = ($0.tag == sender.tag) ? NSControl.StateValue.on : NSControl.StateValue.off }
        m_outgoingSlateDelegate?.onLocationTagSelected(locationTagIndex: sender.tag)
    }

    @IBAction func onTimeRadioButtonClicked(_ sender: NSButton) {
        timeRadioButtons.forEach { $0.state = ($0.tag == sender.tag) ? NSControl.StateValue.on : NSControl.StateValue.off }
        m_outgoingSlateDelegate?.onTimeTagSelected(timeTagIndex: sender.tag)
    }

	//==================================================
	//	IncomingSlateToUIDelegate methods
	//==================================================
	func onReelReceived(_ reelNumber: Int16, _ editable: Bool) {
        reelLabel.stringValue = "\(reelNumber)"

		reelPrevButton.isEnabled = editable
		reelNextButton.isEnabled = editable
    }

    func onSceneTagsReceived(_ sceneTagIndex: Int, _ locationTagIndex: Int, _ timeTagIndex: Int) {
        sceneTagButtons.forEach { $0.state = ($0.tag == sceneTagIndex) ? NSControl.StateValue.on : NSControl.StateValue.off }
        locationRadioButtons.forEach { $0.state = ($0.tag == locationTagIndex) ? NSControl.StateValue.on : NSControl.StateValue.off }
        timeRadioButtons.forEach { $0.state = ($0.tag == timeTagIndex) ? NSControl.StateValue.on : NSControl.StateValue.off }
    }

    func onSceneReceived(_ scene: String) {
        sceneLabel.stringValue = scene
    }

    func onTakeReceived(_ takeNumber: Int8, _ takeTagIndex: Int) {
        takeLabel.stringValue = "\(takeNumber)"
        takeTagButtons.forEach { $0.state = ($0.tag == takeTagIndex) ? NSControl.StateValue.on : NSControl.StateValue.off }
    }

    func onGoodTakeReceived(_ goodTake: Bool) {
        goodTakeButton.state = goodTake ? NSControl.StateValue.on : NSControl.StateValue.off
    }

    func onSlateForNameReceived(_ slateForName: String) {
        sceneName.stringValue = slateForName
    }

    func setSlateForNextClip(_ isSlateForNextClip: Bool) {
        let localizableString = isSlateForNextClip ? "Button.GoodTakeLastClip" : "Button.GoodTake"
        goodTakeButton.title = String.Localized(localizableString)
    }

    func onReelModifiable(_ modifiable: Bool) {
        reelPrevButton.isEnabled = modifiable
        reelNextButton.isEnabled = modifiable
    }

}
