/* -LICENSE-START-
** Copyright (c) 2018 Blackmagic Design
**
** Permission is hereby granted, free of charge, to any person or organization
** obtaining a copy of the software and accompanying documentation covered by
** this license (the "Software") to use, reproduce, display, distribute,
** execute, and transmit the Software, and to prepare derivative works of the
** Software, and to permit third-parties to whom the Software is furnished to
** do so, all subject to the following:
**
** The copyright notices in the Software and this entire statement, including
** the above license grant, this restriction and the following disclaimer,
** must be included in all copies of the Software, in whole or in part, and
** all derivative works of the Software, unless such copies or derivative
** works are solely in the form of machine-executable object code generated by
** a source language processor.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
** DEALINGS IN THE SOFTWARE.
** -LICENSE-END-
*/

import Cocoa
import Utility
import CCU
import BluetoothControl

class ViewController: BasicCameraControlViewController {
	
	@IBOutlet weak var recordButton: NSButton!
	@IBOutlet weak var powerButton: NSButton!
	@IBOutlet weak var timecodeLabel: NSTextField!
	
	var recording: Bool = false
	var powerOn: Bool = false
	
	override func viewDidLoad() {
		super.viewDidLoad()
	}
	
	//=======================================================
	// @IBAction methods
	//=======================================================
	@IBAction func onPowerButtonClicked(_ sender: Any) {
		// Send a power command, on or off.
		sendPowerCommand(powerOn: !powerOn)
	}
	
	@IBAction func onRecordButtonClicked(_ sender: Any) {
		// Create a TransportInfo instance, and set transportMode to Record or Preview (also supports Play).
		var transportInfo = TransportInfo()
		transportInfo.transportMode = recording ? CCUPacketTypes.MediaTransportMode.Preview : CCUPacketTypes.MediaTransportMode.Record
		
		// Create a TransportInfo Command
		if let command: CCUPacketTypes.Command = CCUEncodingFunctions.CreateTransportInfoCommand(info: transportInfo) {
			// Send the command
			sendCCUCommand(data: command.serialize())
		}
	}
	
	//=======================================================
	// BasicCameraControlViewController methods
	//=======================================================
	override func updateDiscoveredPeripheralList(_ discoveredPeripheralList: [DiscoveredPeripheral]) {
		if canConnect() {
			// Connect to the first camera discovered.
			if discoveredPeripheralList.count > 0 {
				let peripheral = discoveredPeripheralList[0].peripheral
				connect(to: peripheral)
			}
		
			// Or connect to a specific camera by name (corresponds to the name
			// displayed in your camera's Bluetooth setup menu).
			/*
			for peripheral in discoveredPeripheralList {
				if peripheral.name.contains("A:2E8945C9") {
					connect(to: peripheral.peripheral)
					break
				}
			}
			*/
		}
	}
	
	override func onCameraReady() {
		// We are connected and the camera is ready to receive commands.
		// Enable the input so we can send commands.
		recordButton.isEnabled = true
		powerButton.isEnabled = true
	}

	// The following methods notify us when we receive a command from the camera.
	// For more commands/methods to override, see the list in BasicCameraControlViewController.swift
	// beneath the label 'PacketDecodedDelegate methods'.

	override func onTransportInfoReceived(_ transportInfo: TransportInfo) {
		recording = (transportInfo.transportMode == CCUPacketTypes.MediaTransportMode.Record)
		recordButton.title = recording ? "Stop" : "Record"
	}
	
	override func onPowerMessageReceived(powerOn: Bool) {
		self.powerOn = powerOn
		powerButton.title = powerOn ? "Power Off" : "Power On"
	}
	
	override func onTimecodeMessageReceived(timecode: Timecode) {
		timecodeLabel.stringValue = timecode.stringValue
	}
}
