/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */

import Foundation

public struct Logger {
    public static func Log(_ message: String) {
        print(message)
    }

    public static func LogWithInfo(_ message: String, filename: String = #file, line: Int = #line, function _: String = #function) {
        Log("[\((filename as NSString).lastPathComponent): \(line)]: \(message)")
    }

    public static func LogWarning(_ message: String, filename: String = #file, line: Int = #line, function: String = #function) {
        LogWithInfo("Warning: \(message)", filename: filename, line: line, function: function)
    }

    public static func LogNoPacketDataWarning(packetType: String, filename: String = #file, line: Int = #line, function: String = #function) {
        LogWithInfo("Warning: Received a \(packetType) packet with no data. Is this a problem?", filename: filename, line: line, function: function)
    }

    public static func LogError(_ message: String, filename: String = #file, line: Int = #line, function: String = #function) {
        LogWithInfo("Error: \(message)", filename: filename, line: line, function: function)
    }

    public static func LogPayloadNoDataError(filename _: String = #file, line _: Int = #line, function _: String = #function) {
        LogError("No data in payload.")
    }
}
